import json
import asyncio
from pyscoresaber import ScoreSaberAPI, ScoreSort

import logging
logging.basicConfig(
    format='%(asctime)s %(levelname)s: %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S',
    level=logging.DEBUG
)
logger = logging.getLogger(__name__)


def filter_and_sort_scores_by_stars(scores, min_stars: float, max_stars: float):
    # Exclude scores outside the specified star range
    filtered_scores = [score for score in scores if min_stars <= score.leaderboard.stars < max_stars]
    # Sort the remaining scores by stars
    sorted_scores = sorted(filtered_scores, key=lambda x: x.leaderboard.stars)
    return sorted_scores

def filter_and_sort_scores_by_acc(scores, min_stars: float, max_stars: float):
    # Exclude scores outside the specified star range
    filtered_scores = [score for score in scores if min_stars <= score.leaderboard.stars < max_stars]
    # Calculate the score percentage for sorting
    sorted_scores = sorted(filtered_scores, key=lambda x: (x.score.base_score / x.leaderboard.max_score) * 100)
    return sorted_scores

# TODO: combine with scores_to_playlist()
def leaderboards_to_playlist(leaderboards: list, playlist_title: str, playlist_author: str = "SaberList Tool") -> str:
    playlist = {
        "playlistTitle": playlist_title,
        "playlistAuthor": playlist_author,
        "songs": []
    }
    
    for leaderboard in leaderboards:
        song_entry = {
            "hash": leaderboard.song_hash,
            "songName": leaderboard.song_name,
            "difficulties": [
                {
                    "name": leaderboard.difficulty.difficulty.name.lower(),
                    "characteristic": leaderboard.difficulty.game_mode.name.lower()
                }
            ],
            "levelAuthorName": leaderboard.level_author_name,
            "songSubName": leaderboard.song_sub_name,
            "songAuthorName": leaderboard.song_author_name
        }
        playlist["songs"].append(song_entry)
    
    playlist_json = json.dumps(playlist, indent=4)
    with open(f"{playlist_title}.bplist", 'w') as file:
        file.write(playlist_json)
    logging.info(f"Playlist written to {playlist_title}.bplist")
    return playlist_json

def scores_to_playlist(scores: list, playlist_title, playlist_author="SaberList Tool", song_limit=0):
    playlist = {
        "playlistTitle": playlist_title,
        "playlistAuthor": playlist_author,
        "songs": []
    }

    # Determine the number of songs to include
    num_songs = len(scores) if song_limit == 0 else min(song_limit, len(scores))

    for score in scores[:num_songs]:
        song_entry = {
            "hash": score.leaderboard.song_hash,
            "songName": score.leaderboard.song_name,
            "difficulties": [
                {
                    "name": score.leaderboard.difficulty.difficulty.name.lower(),
                    "characteristic": score.leaderboard.difficulty.game_mode.name.lower()
                }
            ],
            "levelAuthorName": score.leaderboard.level_author_name
        }
        playlist["songs"].append(song_entry)
    
    playlist_json = json.dumps(playlist, indent=4)
    with open(f"{playlist_title}.bplist", 'w') as file:
        file.write(playlist_json)
    logging.info(f"Playlist written to {playlist_title}.bplist with {num_songs} songs")

    return playlist_json

async def async_replay_all_by_acc():
    """Generate playlists for each star level and sort by lowest accuracy."""

    scoresaber = ExtendedScoreSaberAPI()
    try:
        await scoresaber.start()
        score_sort = ScoreSort.TOP
        scores = []
        default_title = "Replay SS"
        default_player_id = '76561199407393962'
        player_id = input(f"Enter the playerid (Default: {default_player_id}): ") or default_player_id
        song_limit = input("Limit the playlist but number of songs (Default: 0, ie. no limit): ") or 0

        async for player_scores in scoresaber.player_scores_all(player_id, score_sort):
            scores.extend(player_scores)

        if scores:
            max_stars = max(score.leaderboard.stars for score in scores)
        
            for stars in range(1, int(max_stars) + 1):
                playlist_title = f"{default_title} {stars}★"
                filtered_sorted_scores = filter_and_sort_scores_by_acc(scores, stars, stars + 1)
                if filtered_sorted_scores:
                    scores_to_playlist(filtered_sorted_scores, playlist_title, song_limit=int(song_limit))
                else:
                    print(f"No scores found for {stars}★")
        else:
            print("No scores found for the given player.")
        
    finally:
        await scoresaber._http_client.close()

def replay_all_by_acc():
    try:
        asyncio.run(async_replay_all_by_acc())
    except Exception as e:
        print(f"An error occurred: {e}")


async def async_replay_ranked():
    scoresaber = ExtendedScoreSaberAPI()
    try:
        await scoresaber.start()
        
        score_sort = ScoreSort.TOP
        scores = []
        default_player_id = '76561199407393962'
        player_id = input(f"Enter the playerid (Default: {default_player_id}): ") or default_player_id
        default_min_stars = 5
        min_stars = float(input(f"Enter the minimum starlevel to include on the playlist (Default: {default_min_stars}): ") or default_min_stars)
        default_max_stars = min_stars + 1
        max_stars = float(input(f"Enter the maximum starlevel to include on the playlist (Default: {default_max_stars}): ") or default_max_stars)
        default_title = f"Replay SS {min_stars}★"
        playlist_title = input(f"Enter the filename for the playlist (Default: {default_title}): ") or default_title

        async for player_scores in scoresaber.player_scores_all(player_id, score_sort):
            scores.extend(player_scores)

    #   filtered_sorted_scores = filter_and_sort_scores_by_stars(scores, min_stars, max_stars)
        filtered_sorted_scores = filter_and_sort_scores_by_acc(scores, min_stars, max_stars)
        scores_to_playlist(filtered_sorted_scores, playlist_title)
    finally:
        await scoresaber._http_client.close()

def replay_ranked():
    try:
        asyncio.run(async_replay_ranked())
    except Exception as e:
        print(f"An error occurred: {e}")

def leaderboard_songs():

    default_min_stars = 6.0
    min_stars = float(input(f"Enter the minimum starlevel to include on the playlist (Default: {default_min_stars}): ") or default_min_stars)
    default_max_stars = min_stars + 0.10
    max_stars = float(input(f"Enter the maximum starlevel to include on the playlist (Default: {default_max_stars}): ") or default_max_stars)
    default_title = f"SS{min_stars}★"
    playlist_title = input(f"Enter the filename for the playlist (Default: {default_title}): ") or default_title

    try:
        leaderboards = asyncio.run(async_leaderboards(min_stars, max_stars))
    except Exception as e:
        print(f"An error occurred: {e}")

    leaderboards_to_playlist(leaderboards, playlist_title)

async def async_leaderboards(min_stars: float, max_stars: float):
    params = {
        "category": 3,  # sort by scores
        "sort": 1,  # sort ascending
        "max_star": max_stars,
        "min_star": min_stars,
        "qualified": 0, # False
        "ranked": 1 # True
    }
    scoresaber = ExtendedScoreSaberAPI()
    await scoresaber.start()
    all_leaderboards = []
    try:
        async for leaderboard_page in scoresaber.leaderboards_all(**params):
            all_leaderboards.extend(leaderboard_page)
    except Exception as e:
        print(f"An error occurred: {e}")
    finally:
        await scoresaber.close()
    return all_leaderboards

from math import ceil
from typing import AsyncIterable, List

from pyscoresaber.models import LeaderboardInfo, PlayerScore
class ExtendedScoreSaberAPI(ScoreSaberAPI):
    async def leaderboards_all(self, **params) -> AsyncIterable[List[LeaderboardInfo]]:
        page = 1
        max_page = -1
        while page <= max_page or max_page == -1:
            logging.info(f"Fetching page {page}/{max_page} of leaderboards")
            leaderboards = await self.leaderboards(**params, page=page)

            if max_page == -1:
                max_page = ceil(leaderboards.metadata.total / leaderboards.metadata.items_per_page)
            logging.info(f"Got {len(leaderboards.leaderboards)} songs from leaderboard page {page}")

            yield leaderboards.leaderboards
            page += 1

    async def player_scores_all(self, player_id: int, score_sort: ScoreSort) -> AsyncIterable[List[PlayerScore]]:
        page = 1
        max_page = -1

        while page < max_page or max_page == -1:
            logging.info(f"Fetching page {page}/{max_page} of player scores")
            recent_scores = await self.player_scores(player_id, sort=score_sort, limit=100, page=page)

            if max_page == -1:
                max_page = ceil(recent_scores.metadata.total / 100) + 1
            logging.info(f"Got {len(recent_scores.player_scores)} scores from page {page}")

            yield recent_scores.player_scores
            page += 1
